@@ -46,6 +46,15 @@
  *
  * action=mount_add&mount=/live&server=myserver.com&port=8000
  * action=mount_remove&mount=/live&server=myserver.com&port=8000
+ *
+ * On source client connection, a request can be made to trigger a URL request
+ * to verify the details externally. Post info is
+ *
+ * action=stream_auth&mount=/stream&ip=IP&server=SERVER&port=8000&user=fred&pass=pass
+ *
+ * As admin requests can come in for a stream (eg metadata update) these requests
+ * can be issued while stream is active. For these &admin=1 is added to the POST
+ * details.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -80,6 +89,7 @@ typedef struct {
     char *removeurl;
     char *stream_start;
     char *stream_end;
+    char *stream_auth;
     char *username;
     char *password;
     char *auth_header;
@@ -98,6 +108,7 @@ static void auth_url_clear(auth_t *self)
 
     INFO0 ("Doing auth URL cleanup");
     url = self->state;
+    self->state = NULL;
     curl_easy_cleanup (url->handle);
     free (url->username);
     free (url->password);
@@ -112,15 +123,17 @@ static void auth_url_clear(auth_t *self)
 }
 
 
+#ifdef CURLOPT_PASSWDFUNCTION
 /* make sure that prompting at the console does not occur */
 static int my_getpass(void *client, char *prompt, char *buffer, int buflen)
 {
     buffer[0] = '\0';
     return 0;
 }
+#endif
 
 
-static int handle_returned_header (void *ptr, size_t size, size_t nmemb, void *stream)
+static size_t handle_returned_header (void *ptr, size_t size, size_t nmemb, void *stream)
 {
     auth_client *auth_user = stream;
     unsigned bytes = size * nmemb;
@@ -154,19 +167,20 @@ static int handle_returned_header (void *ptr, size_t size, size_t nmemb, void *s
 }
 
 /* capture returned data, but don't do anything with it */
-static int handle_returned_data (void *ptr, size_t size, size_t nmemb, void *stream)
+static size_t handle_returned_data (void *ptr, size_t size, size_t nmemb, void *stream)
 {
     return (int)(size*nmemb);
 }
 
 
-static auth_result url_remove_client (auth_client *auth_user)
+static auth_result url_remove_listener (auth_client *auth_user)
 {
     client_t *client = auth_user->client;
     auth_t *auth = client->auth;
     auth_url *url = auth->state;
     time_t duration = time(NULL) - client->con->con_time;
     char *username, *password, *mount, *server;
+    const char *mountreq;
     ice_config_t *config;
     int port;
     char *userpwd = NULL, post [4096];
@@ -189,10 +203,10 @@ static auth_result url_remove_client (auth_client *auth_user)
         password = strdup ("");
 
     /* get the full uri (with query params if available) */
-    mount = httpp_getvar (client->parser, HTTPP_VAR_RAWURI);
-    if (mount == NULL)
-        mount = httpp_getvar (client->parser, HTTPP_VAR_URI);
-    mount = util_url_escape (mount);
+    mountreq = httpp_getvar (client->parser, HTTPP_VAR_RAWURI);
+    if (mountreq == NULL)
+        mountreq = httpp_getvar (client->parser, HTTPP_VAR_URI);
+    mount = util_url_escape (mountreq);
 
     snprintf (post, sizeof (post),
             "action=listener_remove&server=%s&port=%d&client=%lu&mount=%s"
@@ -240,13 +254,15 @@ static auth_result url_remove_client (auth_client *auth_user)
 }
 
 
-static auth_result url_add_client (auth_client *auth_user)
+static auth_result url_add_listener (auth_client *auth_user)
 {
     client_t *client = auth_user->client;
     auth_t *auth = client->auth;
     auth_url *url = auth->state;
     int res = 0, port;
-    char *agent, *user_agent, *username, *password;
+    const char *agent;
+    char *user_agent, *username, *password;
+    const char *mountreq;
     char *mount, *ipaddr, *server;
     ice_config_t *config;
     char *userpwd = NULL, post [4096];
@@ -272,10 +288,10 @@ static auth_result url_add_client (auth_client *auth_user)
         password = strdup ("");
 
     /* get the full uri (with query params if available) */
-    mount = httpp_getvar (client->parser, HTTPP_VAR_RAWURI);
-    if (mount == NULL)
-        mount = httpp_getvar (client->parser, HTTPP_VAR_URI);
-    mount = util_url_escape (mount);
+    mountreq = httpp_getvar (client->parser, HTTPP_VAR_RAWURI);
+    if (mountreq == NULL)
+        mountreq = httpp_getvar (client->parser, HTTPP_VAR_URI);
+    mount = util_url_escape (mountreq);
     ipaddr = util_url_escape (client->con->ip);
 
     snprintf (post, sizeof (post),
@@ -441,6 +457,52 @@ static void url_stream_end (auth_client *auth_user)
     return;
 }
 
+static void url_stream_auth (auth_client *auth_user)
+{
+    ice_config_t *config;
+    int port;
+    client_t *client = auth_user->client;
+    auth_url *url = client->auth->state;
+    char *mount, *host, *user, *pass, *ipaddr, *admin="";
+    char post [4096];
+
+    if (strchr (url->stream_auth, '@') == NULL)
+    {
+        if (url->userpwd)
+            curl_easy_setopt (url->handle, CURLOPT_USERPWD, url->userpwd);
+        else
+            curl_easy_setopt (url->handle, CURLOPT_USERPWD, "");
+    }
+    else
+        curl_easy_setopt (url->handle, CURLOPT_USERPWD, "");
+    curl_easy_setopt (url->handle, CURLOPT_URL, url->stream_auth);
+    curl_easy_setopt (url->handle, CURLOPT_POSTFIELDS, post);
+    curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
+    if (strcmp (auth_user->mount, httpp_getvar (client->parser, HTTPP_VAR_URI)) != 0)
+        admin = "&admin=1";
+    mount = util_url_escape (auth_user->mount);
+    config = config_get_config ();
+    host = util_url_escape (config->hostname);
+    port = config->port;
+    config_release_config ();
+    user = util_url_escape (client->username);
+    pass = util_url_escape (client->password);
+    ipaddr = util_url_escape (client->con->ip);
+
+    snprintf (post, sizeof (post),
+            "action=stream_auth&mount=%s&ip=%s&server=%s&port=%d&user=%s&pass=%s%s",
+            mount, ipaddr, host, port, user, pass, admin);
+    free (ipaddr);
+    free (user);
+    free (pass);
+    free (mount);
+    free (host);
+
+    client->authenticated = 0;
+    if (curl_easy_perform (url->handle))
+        WARN2 ("auth to server %s failed with %s", url->stream_auth, url->errormsg);
+}
+
 
 static auth_result auth_url_adduser(auth_t *auth, const char *username, const char *password)
 {
@@ -461,34 +523,61 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
 {
     auth_url *url_info;
 
-    authenticator->authenticate = url_add_client;
-    authenticator->release_client = url_remove_client;
-
     authenticator->free = auth_url_clear;
     authenticator->adduser = auth_url_adduser;
     authenticator->deleteuser = auth_url_deleteuser;
     authenticator->listuser = auth_url_listuser;
 
-    authenticator->stream_start = url_stream_start;
-    authenticator->stream_end = url_stream_end;
-
     url_info = calloc(1, sizeof(auth_url));
+    authenticator->state = url_info;
+
+    /* default headers */
     url_info->auth_header = strdup ("icecast-auth-user: 1\r\n");
     url_info->timelimit_header = strdup ("icecast-auth-timelimit:");
 
+    /* force auth thread to call function. this makes sure the auth_t is attached to client */
+    authenticator->authenticate = url_add_listener;
+
     while(options) {
         if(!strcmp(options->name, "username"))
+        {
+            free (url_info->username);
             url_info->username = strdup (options->value);
+        }
         if(!strcmp(options->name, "password"))
+        {
+            free (url_info->password);
             url_info->password = strdup (options->value);
+        }
         if(!strcmp(options->name, "listener_add"))
+        {
+            free (url_info->addurl);
             url_info->addurl = strdup (options->value);
+        }
         if(!strcmp(options->name, "listener_remove"))
+        {
+            authenticator->release_listener = url_remove_listener;
+            free (url_info->removeurl);
             url_info->removeurl = strdup (options->value);
+        }
         if(!strcmp(options->name, "mount_add"))
+        {
+            authenticator->stream_start = url_stream_start;
+            free (url_info->stream_start);
             url_info->stream_start = strdup (options->value);
+        }
         if(!strcmp(options->name, "mount_remove"))
+        {
+            authenticator->stream_end = url_stream_end;
+            free (url_info->stream_end);
             url_info->stream_end = strdup (options->value);
+        }
+        if(!strcmp(options->name, "stream_auth"))
+        {
+            authenticator->stream_auth = url_stream_auth;
+            free (url_info->stream_auth);
+            url_info->stream_auth = strdup (options->value);
+        }
         if(!strcmp(options->name, "auth_header"))
         {
             free (url_info->auth_header);
@@ -504,7 +593,7 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
     url_info->handle = curl_easy_init ();
     if (url_info->handle == NULL)
     {
-        free (url_info);
+        auth_url_clear (authenticator);
         return -1;
     }
     if (url_info->auth_header)
@@ -518,7 +607,9 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
     curl_easy_setopt (url_info->handle, CURLOPT_WRITEDATA, url_info->handle);
     curl_easy_setopt (url_info->handle, CURLOPT_NOSIGNAL, 1L);
     curl_easy_setopt (url_info->handle, CURLOPT_TIMEOUT, 15L);
+#ifdef CURLOPT_PASSWDFUNCTION
     curl_easy_setopt (url_info->handle, CURLOPT_PASSWDFUNCTION, my_getpass);
+#endif
     curl_easy_setopt (url_info->handle, CURLOPT_ERRORBUFFER, &url_info->errormsg[0]);
 
     if (url_info->username && url_info->password)
@@ -528,7 +619,6 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
         snprintf (url_info->userpwd, len, "%s:%s", url_info->username, url_info->password);
     }
 
-    authenticator->state = url_info;
     INFO0("URL based authentication setup");
     return 0;
 }
