@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011,      Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,12 +30,14 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/socket.h>
+#define SCN_OFF_T SCNdMAX
+#define PRI_OFF_T PRIdMAX
 #else
 #include <winsock2.h>
 #include <windows.h>
 #define fseeko fseek
-#define PRIdMAX "ld"
-#define SCNdMAX "ld"
+#define SCN_OFF_T "ld"
+#define PRI_OFF_T "ld"
 #define snprintf _snprintf
 #define strncasecmp _strnicmp
 #define S_ISREG(mode)  ((mode) & _S_IFREG)
@@ -64,14 +67,13 @@
 
 #define BUFSIZE 4096
 
-static fserve_t *active_list = NULL;
-static volatile fserve_t *pending_list = NULL;
+fserve_t *active_list = NULL;
+fserve_t *pending_list = NULL;
 
-static mutex_t pending_lock;
+static spin_t pending_lock;
 static avl_tree *mimetypes = NULL;
 
-static thread_type *fserv_thread;
-static int run_fserv = 0;
+static volatile int run_fserv = 0;
 static unsigned int fserve_clients;
 static int client_tree_changed=0;
 
@@ -96,28 +98,41 @@ void fserve_initialize(void)
     ice_config_t *config = config_get_config();
 
     mimetypes = NULL;
-    thread_mutex_create (&pending_lock);
+    active_list = NULL;
+    pending_list = NULL;
+    thread_spin_create (&pending_lock);
 
     fserve_recheck_mime_types (config);
     config_release_config();
 
-    run_fserv = 1;
     stats_event (NULL, "file_connections", "0");
-
-    fserv_thread = thread_create("File Serving Thread", 
-            fserv_thread_function, NULL, THREAD_ATTACHED);
+    INFO0("file serving started");
 }
 
 void fserve_shutdown(void)
 {
-    if(!run_fserv)
-        return;
-
+    thread_spin_lock (&pending_lock);
     run_fserv = 0;
-    thread_join(fserv_thread);
-    INFO0("file serving thread stopped");
+    while (pending_list)
+    {
+        fserve_t *to_go = (fserve_t *)pending_list;
+        pending_list = to_go->next;
+
+        fserve_client_destroy (to_go);
+    }
+    while (active_list)
+    {
+        fserve_t *to_go = active_list;
+        active_list = to_go->next;
+        fserve_client_destroy (to_go);
+    }
+
     if (mimetypes)
         avl_tree_free (mimetypes, _delete_mapping);
+
+    thread_spin_unlock (&pending_lock);
+    thread_spin_destroy (&pending_lock);
+    INFO0("file serving stopped");
 }
 
 #ifdef HAVE_POLL
@@ -142,7 +157,12 @@ int fserve_client_waiting (void)
         }
     }
     if (!ufds)
-        thread_sleep(200000);
+    {
+        thread_spin_lock (&pending_lock);
+        run_fserv = 0;
+        thread_spin_unlock (&pending_lock);
+        return -1;
+    }
     else if (poll(ufds, fserve_clients, 200) > 0)
     {
         /* mark any clients that are ready */
@@ -178,7 +198,12 @@ int fserve_client_waiting (void)
     }
     /* hack for windows, select needs at least 1 descriptor */
     if (fd_max == SOCK_ERROR)
-        thread_sleep (200000);
+    {
+        thread_spin_lock (&pending_lock);
+        run_fserv = 0;
+        thread_spin_unlock (&pending_lock);
+        return -1;
+    }
     else
     {
         struct timeval tv;
@@ -204,15 +229,17 @@ int fserve_client_waiting (void)
 }
 #endif
 
-static void wait_for_fds(void) {
+static int wait_for_fds(void)
+{
     fserve_t *fclient;
+    int ret;
 
     while (run_fserv)
     {
         /* add any new clients here */
         if (pending_list)
         {
-            thread_mutex_lock (&pending_lock);
+            thread_spin_lock (&pending_lock);
 
             fclient = (fserve_t*)pending_list;
             while (fclient)
@@ -225,12 +252,14 @@ static void wait_for_fds(void) {
                 fserve_clients++;
             }
             pending_list = NULL;
-            thread_mutex_unlock (&pending_lock);
+            thread_spin_unlock (&pending_lock);
         }
         /* drop out of here if someone is ready */
-        if (fserve_client_waiting())
-            break;
+        ret = fserve_client_waiting();
+        if (ret)
+            return ret;
     }
+    return -1;
 }
 
 static void *fserv_thread_function(void *arg)
@@ -238,9 +267,10 @@ static void *fserv_thread_function(void *arg)
     fserve_t *fclient, **trail;
     size_t bytes;
 
-    INFO0("file serving thread started");
-    while (run_fserv) {
-        wait_for_fds();
+    while (1)
+    {
+        if (wait_for_fds() < 0)
+            break;
 
         fclient = active_list;
         trail = &active_list;
@@ -272,8 +302,10 @@ static void *fserv_thread_function(void *arg)
                             client_tree_changed = 1;
                             continue;
                         }
-                        client_set_queue (client, refbuf->next);
-                        refbuf = client->refbuf;
+                        refbuf = refbuf->next;
+                        client->refbuf->next = NULL;
+                        refbuf_release (client->refbuf);
+                        client->refbuf = refbuf;
                         bytes = refbuf->len;
                     }
                     refbuf->len = (unsigned int)bytes;
@@ -298,25 +330,7 @@ static void *fserv_thread_function(void *arg)
             fclient = fclient->next;
         }
     }
-
-    /* Shutdown path */
-    thread_mutex_lock (&pending_lock);
-    while (pending_list)
-    {
-        fserve_t *to_go = (fserve_t *)pending_list;
-        pending_list = to_go->next;
-
-        fserve_client_destroy (to_go);
-    }
-    thread_mutex_unlock (&pending_lock);
-
-    while (active_list)
-    {
-        fserve_t *to_go = active_list;
-        active_list = to_go->next;
-        fserve_client_destroy (to_go);
-    }
-
+    DEBUG0 ("fserve handler exit");
     return NULL;
 }
 
@@ -328,7 +342,7 @@ char *fserve_content_type (const char *path)
     void *result;
     char *type;
 
-    thread_mutex_lock (&pending_lock);
+    thread_spin_lock (&pending_lock);
     if (mimetypes && !avl_get_by_key (mimetypes, &exttype, &result))
     {
         mime_type *mime = result;
@@ -357,7 +371,7 @@ char *fserve_content_type (const char *path)
         else
             type = strdup ("application/octet-stream");
     }
-    thread_mutex_unlock (&pending_lock);
+    thread_spin_unlock (&pending_lock);
     return type;
 }
 
@@ -392,32 +406,36 @@ int fserve_client_create (client_t *httpclient, const char *path)
     int ret = 0;
     char *fullpath;
     int m3u_requested = 0, m3u_file_available = 1;
-    int xspf_requested = 0, xspf_file_available = 1;
+    const char * xslt_playlist_requested = NULL;
+    int xslt_playlist_file_available = 1;
     ice_config_t *config;
     FILE *file;
 
     fullpath = util_get_path_from_normalised_uri (path);
-    INFO2 ("checking for file %s (%s)", path, fullpath);
+    INFO2 ("checking for file %H (%H)", path, fullpath);
 
     if (strcmp (util_get_extension (fullpath), "m3u") == 0)
         m3u_requested = 1;
 
     if (strcmp (util_get_extension (fullpath), "xspf") == 0)
-        xspf_requested = 1;
+        xslt_playlist_requested = "xspf.xsl";
+
+    if (strcmp (util_get_extension (fullpath), "vclt") == 0)
+        xslt_playlist_requested = "vclt.xsl";
 
     /* check for the actual file */
     if (stat (fullpath, &file_buf) != 0)
     {
         /* the m3u can be generated, but send an m3u file if available */
-        if (m3u_requested == 0 && xspf_requested == 0)
+        if (m3u_requested == 0 && xslt_playlist_requested == NULL)
         {
-            WARN2 ("req for file \"%s\" %s", fullpath, strerror (errno));
+            WARN2 ("req for file \"%H\" %s", fullpath, strerror (errno));
             client_send_404 (httpclient, "The file you requested could not be found");
             free (fullpath);
             return -1;
         }
         m3u_file_available = 0;
-        xspf_file_available = 0;
+        xslt_playlist_file_available = 0;
     }
 
     httpclient->refbuf->len = PER_CLIENT_REFBUF_SIZE;
@@ -464,16 +482,16 @@ int fserve_client_create (client_t *httpclient, const char *path)
         free (fullpath);
         return 0;
     }
-    if (xspf_requested && xspf_file_available == 0)
+    if (xslt_playlist_requested && xslt_playlist_file_available == 0)
     {
         xmlDocPtr doc;
         char *reference = strdup (path);
         char *eol = strrchr (reference, '.');
         if (eol)
             *eol = '\0';
-        stats_get_xml (&doc, 0, reference);
+        doc = stats_get_xml (0, reference);
         free (reference);
-        admin_send_response (doc, httpclient, TRANSFORMED, "xspf.xsl");
+        admin_send_response (doc, httpclient, TRANSFORMED, xslt_playlist_requested);
         xmlFreeDoc(doc);
         return 0;
     }
@@ -482,7 +500,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     config = config_get_config();
     if (config->fileserve == 0)
     {
-        DEBUG1 ("on demand file \"%s\" refused", fullpath);
+        DEBUG1 ("on demand file \"%H\" refused", fullpath);
         client_send_404 (httpclient, "The file you requested could not be found");
         config_release_config();
         free (fullpath);
@@ -493,7 +511,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     if (S_ISREG (file_buf.st_mode) == 0)
     {
         client_send_404 (httpclient, "The file you requested could not be found");
-        WARN1 ("found requested file but there is no handler for it: %s", fullpath);
+        WARN1 ("found requested file but there is no handler for it: %H", fullpath);
         free (fullpath);
         return -1;
     }
@@ -501,7 +519,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     file = fopen (fullpath, "rb");
     if (file == NULL)
     {
-        WARN1 ("Problem accessing file \"%s\"", fullpath);
+        WARN1 ("Problem accessing file \"%H\"", fullpath);
         client_send_404 (httpclient, "File not readable");
         free (fullpath);
         return -1;
@@ -515,7 +533,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     if (range != NULL) {
         ret = 0;
         if (strncasecmp (range, "bytes=", 6) == 0)
-            ret = sscanf (range+6, "%" SCNdMAX "-", &rangenumber);
+            ret = sscanf (range+6, "%" SCN_OFF_T "-", &rangenumber);
 
         if (ret != 1) {
             /* format not correct, so lets just assume
@@ -557,9 +575,9 @@ int fserve_client_create (client_t *httpclient, const char *path)
                     "HTTP/1.1 206 Partial Content\r\n"
                     "Date: %s\r\n"
                     "Accept-Ranges: bytes\r\n"
-                    "Content-Length: %" PRIdMAX "\r\n"
-                    "Content-Range: bytes %" PRIdMAX \
-                    "-%" PRIdMAX "/%" PRIdMAX "\r\n"
+                    "Content-Length: %" PRI_OFF_T "\r\n"
+                    "Content-Range: bytes %" PRI_OFF_T \
+                    "-%" PRI_OFF_T "/%" PRI_OFF_T "\r\n"
                     "Content-Type: %s\r\n\r\n",
                     currenttime,
                     new_content_len,
@@ -583,7 +601,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
         bytes = snprintf (httpclient->refbuf->data, BUFSIZE,
             "HTTP/1.0 200 OK\r\n"
             "Accept-Ranges: bytes\r\n"
-            "Content-Length: %" PRIdMAX "\r\n"
+            "Content-Length: %" PRI_OFF_T "\r\n"
             "Content-Type: %s\r\n\r\n",
             content_length,
             type);
@@ -607,6 +625,24 @@ fail:
 }
 
 
+/* Routine to actually add pre-configured client structure to pending list and
+ * then to start off the file serving thread if it is not already running
+ */
+static void fserve_add_pending (fserve_t *fclient)
+{
+    thread_spin_lock (&pending_lock);
+    fclient->next = (fserve_t *)pending_list;
+    pending_list = fclient;
+    if (run_fserv == 0)
+    {
+        run_fserv = 1;
+        DEBUG0 ("fserve handler waking up");
+        thread_create("File Serving Thread", fserv_thread_function, NULL, THREAD_DETACHED);
+    }
+    thread_spin_unlock (&pending_lock);
+}
+
+
 /* Add client to fserve thread, client needs to have refbuf set and filled
  * but may provide a NULL file if no data needs to be read
  */
@@ -623,11 +659,7 @@ int fserve_add_client (client_t *client, FILE *file)
     fclient->file = file;
     fclient->client = client;
     fclient->ready = 0;
-
-    thread_mutex_lock (&pending_lock);
-    fclient->next = (fserve_t *)pending_list;
-    pending_list = fclient;
-    thread_mutex_unlock (&pending_lock);
+    fserve_add_pending (fclient);
 
     return 0;
 }
@@ -652,10 +684,7 @@ void fserve_add_client_callback (client_t *client, fserve_callback_t callback, v
     fclient->callback = callback;
     fclient->arg = arg;
 
-    thread_mutex_lock (&pending_lock);
-    fclient->next = (fserve_t *)pending_list;
-    pending_list = fclient;
-    thread_mutex_unlock (&pending_lock);
+    fserve_add_pending (fclient);
 }
 
 
@@ -737,10 +766,10 @@ void fserve_recheck_mime_types (ice_config_t *config)
     }
     fclose(mimefile);
 
-    thread_mutex_lock (&pending_lock);
+    thread_spin_lock (&pending_lock);
     if (mimetypes)
         avl_tree_free (mimetypes, _delete_mapping);
     mimetypes = new_mimetypes;
-    thread_mutex_unlock (&pending_lock);
+    thread_spin_unlock (&pending_lock);
 }
 
