@@ -17,10 +17,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
+
+#ifdef WIN32
+#define _WIN32_WINNT 0x0400
+/* For getpid() */
+#include <process.h>
+#include <windows.h>
+#define snprintf _snprintf
+#define getpid _getpid
+#endif
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
+#ifdef HAVE_CURL
+#include <curl/curl.h>
+#endif
 
 #include "thread/thread.h"
 #include "avl/avl.h"
@@ -32,14 +45,13 @@
 #include <sys/types.h>
 #include <grp.h>
 #include <pwd.h>
-#include <errno.h>
 #endif
 
 #include "cfgfile.h"
 #include "sighandler.h"
 
 #include "global.h"
-#include "os.h"
+#include "compat.h"
 #include "connection.h"
 #include "refbuf.h"
 #include "client.h"
@@ -53,13 +65,6 @@
 
 #include <libxml/xmlmemory.h>
 
-#ifdef _WIN32
-/* For getpid() */
-#include <process.h>
-#define snprintf _snprintf
-#define getpid _getpid
-#endif
-
 #undef CATMODULE
 #define CATMODULE "main"
 
@@ -68,16 +73,18 @@ static char *pidfile = NULL;
 
 static void _fatal_error(char *perr)
 {
-#ifdef WIN32
+#ifdef WIN32_SERVICE
+    MessageBox(NULL, perr, "Error", MB_SERVICE_NOTIFICATION);
+#elif defined(WIN32)
     MessageBox(NULL, perr, "Error", MB_OK);
 #else
     fprintf(stdout, "%s\n", perr);
 #endif
 }
 
-static void _print_usage()
+static void _print_usage(void)
 {
-    printf(ICECAST_VERSION_STRING "\n\n");
+    printf("%s\n\n", ICECAST_VERSION_STRING);
     printf("usage: icecast [-b -v] -c <file>\n");
     printf("options:\n");
     printf("\t-c <file>\tSpecify configuration file\n");
@@ -93,7 +100,7 @@ static void _stop_logging(void)
     log_close(playlistlog);
 }
 
-static void _initialize_subsystems(void)
+void initialize_subsystems(void)
 {
     log_initialize();
     thread_initialize();
@@ -103,13 +110,16 @@ static void _initialize_subsystems(void)
     connection_initialize();
     global_initialize();
     refbuf_initialize();
+
     xslt_initialize();
+#ifdef HAVE_CURL_GLOBAL_INIT
+    curl_global_init (CURL_GLOBAL_ALL);
+#endif
 }
 
-static void _shutdown_subsystems(void)
+void shutdown_subsystems(void)
 {
     fserve_shutdown();
-    xslt_shutdown();
     refbuf_shutdown();
     slave_shutdown();
     auth_shutdown();
@@ -123,11 +133,14 @@ static void _shutdown_subsystems(void)
     sock_shutdown();
     thread_shutdown();
 
+#ifdef HAVE_CURL
+    curl_global_cleanup();
+#endif
+
     /* Now that these are done, we can stop the loggers. */
     _stop_logging();
     log_shutdown();
-
-    xmlCleanupParser();
+    xslt_shutdown();
 }
 
 static int _parse_config_opts(int argc, char **argv, char *filename, int size)
@@ -261,41 +274,6 @@ static int _start_logging(void)
     return 0;
 }
 
-static int _setup_sockets(void)
-{
-    ice_config_t *config;
-    int i = 0;
-    int ret = 0;
-    int successful = 0;
-    char pbuf[1024];
-
-    config = config_get_config_unlocked();
-
-    for(i = 0; i < MAX_LISTEN_SOCKETS; i++) {
-        if(config->listeners[i].port <= 0)
-            break;
-
-        global.serversock[i] = sock_get_server_socket(
-                config->listeners[i].port, config->listeners[i].bind_address);
-
-        if (global.serversock[i] == SOCK_ERROR) {
-            memset(pbuf, '\000', sizeof(pbuf));
-            snprintf(pbuf, sizeof(pbuf)-1, 
-                "Could not create listener socket on port %d", 
-                config->listeners[i].port);
-            _fatal_error(pbuf);
-            return 0;
-        }
-        else {
-            ret = 1;
-            successful++;
-        }
-    }
-
-    global.server_sockets = successful;
-    
-    return ret;
-}
 
 static int _start_listening(void)
 {
@@ -304,7 +282,7 @@ static int _start_listening(void)
         if (sock_listen(global.serversock[i], ICE_LISTEN_QUEUE) == SOCK_ERROR)
             return 0;
 
-        sock_set_blocking(global.serversock[i], SOCK_NONBLOCK);
+        sock_set_blocking(global.serversock[i], 0);
     }
 
     return 1;
@@ -313,9 +291,9 @@ static int _start_listening(void)
 /* bind the socket and start listening */
 static int _server_proc_init(void)
 {
-    ice_config_t *config;
+    ice_config_t *config = config_get_config_unlocked();
 
-    if (!_setup_sockets())
+    if (connection_setup_sockets (config) < 1)
         return 0;
 
     if (!_start_listening()) {
@@ -323,7 +301,6 @@ static int _server_proc_init(void)
         return 0;
     }
 
-    config = config_get_config_unlocked();
     /* recreate the pid file */
     if (config->pidfile)
     {
@@ -342,8 +319,6 @@ static int _server_proc_init(void)
 /* this is the heart of the beast */
 static void _server_proc(void)
 {
-    int i;
-
     if (background)
     {
         fclose (stdin);
@@ -352,8 +327,7 @@ static void _server_proc(void)
     }
     connection_accept_loop();
 
-    for(i=0; i < MAX_LISTEN_SOCKETS; i++)
-        sock_close(global.serversock[i]);
+    connection_setup_sockets (NULL);
 }
 
 /* chroot the process. Watch out - we need to do this before starting other
@@ -448,9 +422,10 @@ int main(int argc, char **argv)
     */
     res = _parse_config_opts(argc, argv, filename, 512);
     if (res == 1) {
+#if !defined(_WIN32) || defined(_CONSOLE)
         /* startup all the modules */
-        _initialize_subsystems();
-
+        initialize_subsystems();
+#endif
         /* parse the config file */
         config_get_config();
         ret = config_initial_parse_file(filename);
@@ -474,7 +449,9 @@ int main(int argc, char **argv)
                 _fatal_error("XML config parsing error");
                 break;
             }
-            _shutdown_subsystems();
+#if !defined(_WIN32) || defined(_CONSOLE)
+            shutdown_subsystems();
+#endif
             return 1;
         }
     } else if (res == -1) {
@@ -488,7 +465,7 @@ int main(int argc, char **argv)
     /* Bind socket, before we change userid */
     if(!_server_proc_init()) {
         _fatal_error("Server startup failed. Exiting");
-        _shutdown_subsystems();
+        shutdown_subsystems();
         return 1;
     }
 
@@ -504,7 +481,7 @@ int main(int argc, char **argv)
     {
         fprintf(stderr, "ERROR: You should not run icecast2 as root\n");
         fprintf(stderr, "Use the changeowner directive in the config file\n");
-        _shutdown_subsystems();
+        shutdown_subsystems();
         return 1;
     }
 #endif
@@ -514,11 +491,11 @@ int main(int argc, char **argv)
 
     if (!_start_logging()) {
         _fatal_error("FATAL: Could not start logging");
-        _shutdown_subsystems();
+        shutdown_subsystems();
         return 1;
     }
 
-    INFO0 (ICECAST_VERSION_STRING " server started");
+    INFO1 ("%s server started", ICECAST_VERSION_STRING);
 
     /* REM 3D Graphics */
 
@@ -535,9 +512,9 @@ int main(int argc, char **argv)
     _server_proc();
 
     INFO0("Shutting down");
-
-    _shutdown_subsystems();
-
+#if !defined(_WIN32) || defined(_CONSOLE)
+    shutdown_subsystems();
+#endif
     if (pidfile)
     {
         remove (pidfile);
