@@ -51,6 +51,7 @@ typedef struct source_tag
 
     unsigned long peak_listeners;
     unsigned long listeners;
+    unsigned long prev_listeners;
     long max_listeners;
     int yp_public;
     int fallback_override;
@@ -79,6 +80,7 @@ typedef struct source_tag
 
 source_t *source_reserve (const char *mount);
 void *source_client_thread (void *arg);
+void source_startup (client_t *client, const char *uri, int auth_style);
 void source_client_callback (client_t *client, void *source);
 void source_update_settings (ice_config_t *config, source_t *source, mount_proxy *mountinfo);
 void source_clear_source (source_t *source);
@@ -90,7 +92,7 @@ void source_free_source(source_t *source);
 void source_move_clients (source_t *source, source_t *dest);
 int source_remove_client(void *key);
 void source_main(source_t *source);
-void source_recheck_mounts (void);
+void source_recheck_mounts (int update_all);
 
 extern mutex_t move_clients_mutex;
 
